"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.gc = gc;
exports.protectVTree = protectVTree;
exports.unprotectVTree = unprotectVTree;

var _pool = _interopRequireDefault(require("./pool"));

var _types = require("./types");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const {
  protect,
  unprotect,
  memory
} = _pool.default;
/**
 * Ensures that vTree is not recycled during a render cycle.
 *
 * @param {VTree} vTree
 * @return {void}
 */

function protectVTree(vTree) {
  protect(vTree);

  if (vTree.childNodes.length) {
    for (let i = 0; i < vTree.childNodes.length; i++) {
      protectVTree(vTree.childNodes[i]);
    }
  }
}
/**
 * Recycles a VTree by unprotecting itself, removing its DOM Node reference, and
 * recursively unprotecting all nested children. Resets the VTree's attributes
 * and childNode properties afterwards, as these can contribute to unwanted
 * increases in the heap.
 *
 * @param {VTree} vTree
 * @return {void}
 */


function unprotectVTree(vTree) {
  if (vTree.childNodes.length) {
    for (let i = 0; i < vTree.childNodes.length; i++) {
      unprotectVTree(vTree.childNodes[i]);
    }
  }

  _types.NodeCache.delete(vTree);

  unprotect(vTree);
}
/**
 * Collects any unused VTree's and puts them back into the free Set. This is
 * primarily used by tests, but could also be useful for specific niche cases
 * as a way to ease memory/CPU pressure when lots of temporary trees are
 * created but never used.
 *
 * @return {void}
 */


function gc() {
  memory.allocated.forEach(vTree => {
    memory.free.add(vTree);
    memory.allocated.delete(vTree);

    _types.NodeCache.delete(vTree);
  });
}